@@ -152,6 +152,9 @@ func (s *server) Initialize(
 			DocumentFormattingProvider: true,
 			HoverProvider:              true,
 			ReferencesProvider:         &protocol.ReferenceOptions{},
+			RenameProvider: &protocol.RenameOptions{
+				PrepareProvider: true,
+			},
 			SemanticTokensProvider: &SemanticTokensOptions{
 				Legend: SemanticTokensLegend{
 					TokenTypes:     semanticTypeLegend,
@@ -559,6 +562,38 @@ func (s *server) DocumentSymbol(ctx context.Context, params *protocol.DocumentSy
 	return anyResults, nil
 }
 
+// PrepareRename is the entry point for checking workspace wide renaming of a symbol.
+//
+// If a symbol can be renamed, PrepareRename will return the range for the rename. Returning
+// an empty range indicates that the requested position cannot be renamed and the client
+// will handle providing feedback to the user.
+//
+// Supported symbol types for renaming are [referenceable], [static], and [reference].
+func (s *server) PrepareRename(ctx context.Context, params *protocol.PrepareRenameParams) (*protocol.Range, error) {
+	symbol := s.getSymbol(ctx, params.TextDocument.URI, params.Position)
+	if symbol == nil {
+		return nil, nil
+	}
+	switch symbol.kind.(type) {
+	case *referenceable, *static, *reference:
+		rnge := reportSpanToProtocolRange(symbol.span)
+		return &rnge, nil
+	}
+	return nil, nil
+}
+
+// Rename is the entry point for workspace wide renaming of a symbol.
+func (s *server) Rename(
+	ctx context.Context,
+	params *protocol.RenameParams,
+) (*protocol.WorkspaceEdit, error) {
+	symbol := s.getSymbol(ctx, params.TextDocument.URI, params.Position)
+	if symbol == nil {
+		return nil, nil
+	}
+	return symbol.Rename(params.NewName)
+}
+
 // getSymbol is a helper function that gets the *[symbol] for the given [protocol.URI] and
 // [protocol.Position].
 func (s *server) getSymbol(
